@@ -1,3 +1,5 @@
+# -*- coding: iso-8859-1 -*-
+
 # GNU Solfege - ear training for GNOME
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
 #
@@ -15,22 +17,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import __builtin__
-import types
-try:
-    basestring
-except:
-    basestring = (types.StringType, types.UnicodeType)
-    __builtin__.basestring = basestring
+copyright = "Copyright © 1999-2005 Tom Cato Amundsen <tca@gnu.org>"
 
+import __builtin__
 __builtin__.app_running = True
 __builtin__.__exdata__ = []
 
 import sys, os, time, random
 
-if sys.version_info < (2, 2):
+if sys.version_info < (2, 3):
     print "********************************************************************"
-    print "Solfege need Python 2.2 or newer. The configure script told you so!"
+    print "Solfege need Python 2.3 or newer. The configure script told you so!"
     print "********************************************************************"
     sys.exit(-1)
 
@@ -41,35 +38,38 @@ i18n.setup()
 
 import configureoutput
 
-def do_some_cmdline_options(argv):
-    if '-h' in argv or '--help' in argv:
-        print """
-GNU Solfege %s
-Type "man solfege" for more details.
-Options:
-  --no-sound             Do not play any sounds. Useful when porting.
-  --verbose-sound-init   display more info about the sound setup
-  -v, --version
-  -w, --warranty         show warranty and copyright
-  -h, --help             Display this message
-  --gtk-help             Display the usual gtk/gnome commandline options.
-    """ % configureoutput.VERSION_STRING
-        sys.exit()
+import optparse
 
-    if '-v' in argv or '--version' in argv:
-        print """GNU Solfege %s
+opt_parser = optparse.OptionParser(usage='GNU Solfege %s\nType "man solfege" for more details.' % configureoutput.VERSION_STRING )
+opt_parser.add_option('-v', '--version', action='store_true', dest='version')
+opt_parser.add_option('-w', '--warranty', action='store_true', dest='warranty',
+    help='show warranty and copyright')
+opt_parser.add_option('--no-splash', action='store_false', dest='no_splash',
+    default=False)
+opt_parser.add_option('--verbose-sound-init', action='store_true',
+    default=False,
+    dest='verbose_sound_init', help='Display more info about the sound setup')
+opt_parser.add_option('--no-sound', action='store_true', dest='no_sound',
+    default=False,
+    help='Do not play any sounds. Useful when porting.')
+opt_parser.add_option('--no-random', action='store_true', dest='no_random',
+    help='For debugging only: Select questions from lesson files in sequential order.')
+
+options, args = opt_parser.parse_args()
+
+if options.version:
+    print """GNU Solfege %s
 This is free software. It is covered by the GNU General Public License,
 and you are welcome to change it and/or distribute copies of it under
 certain conditions. Invoke as `solfege --warranty` for more information.
 
-Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen <tca@gnu.org>
-        """ % configureoutput.VERSION_STRING
-        sys.exit()
-
-    if '-w' in argv or '--warranty' in argv:
-        print """GNU Solfege %s
-Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen <tca@gnu.org>
+%s
+        """ % (configureoutput.VERSION_STRING, copyright)
+    sys.exit()
 
+if options.warranty:
+    print """GNU Solfege %s
+%s
     This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License version 2
 as published by the Free Software Foundation.
@@ -83,15 +83,8 @@ General Public License for more details.
 GNU General Public License along with this program; if not, write to
 the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 USA.
-        """ % configureoutput.VERSION_STRING
-        sys.exit()
-
-
-do_some_cmdline_options(sys.argv)
-
-if '--gtk-help' in sys.argv:
-    del sys.argv[sys.argv.index('--gtk-help')]
-    sys.argv.append('--help')
+    """ % (configureoutput.VERSION_STRING, copyright)
+    sys.exit()
 
 # silent, GNOME, be silent!
 sys.argv.append('--disable-sound')
@@ -142,7 +135,7 @@ class SplashWin(gtk.Window):
         while gtk.events_pending():
             gtk.main_iteration(0)
 
-if not filter(lambda a: a in sys.argv, ['--no-splash', '--usage', '-?', '--gtk-help']):
+if options.no_splash:
     splash_win = SplashWin()
     time.sleep(0.1)
     gtk.gdk.flush()
@@ -150,25 +143,17 @@ if not filter(lambda a: a in sys.argv, ['--no-splash', '--usage', '-?', '--gtk-h
         gtk.main_iteration(0)
 else:
     splash_win = None
-    if '--no-splash' in sys.argv:
-        del sys.argv[sys.argv.index('--no-splash')]
-
-# we have to remove all options gnome/gtk don't understand
-# from sys.argv
-args = []
-for a in ('--no-sound', '--verbose-sound-init'):
-    if a in sys.argv:
-        args.append(a)
-        del sys.argv[sys.argv.index(a)]
 
 if splash_win:
     splash_win.show_progress("importing GNOME modules")
 
 import gnome, gnome.ui
-# FIXME we set this because it is not yet added to PyGNOME2
-gnome.ui.PAD = 8
-gnome.ui.PAD_SMALL = 4
-gnome.ui.PAD_BIG = 12
+# FIXME PAD AND PAD_SMALL are missing from some gnome-python releases.
+try:
+    gnome.ui.PAD
+except:
+    gnome.ui.PAD = 8
+    gnome.ui.PAD_SMALL = 4
 
 if splash_win:
     splash_win.show_progress("importing application modules")
@@ -189,26 +174,33 @@ import mpd, mpd.musicdisplayer, gu, cfg
 import stock
 
 
-# The order the modules are imported affect the ordering in the menus.
-import melodicinterval, chord, rhythm
-import chordvoicing, harmonicinterval, \
-       singinterval, idbyname, dictation, twelvetone, \
-       idtone, compareintervals, singchord, identifybpm, \
-       harmonicprogressiondictation, singanswer
+import chord
+import chordvoicing, harmonicinterval, melodicinterval,\
+       singinterval
 
 import exercise_setup
 exercise_setup.register_exercise({
     'exercise_name': 'id-by-name',
-    'exercise_config': 'solfege/scales',
-    'menu_path': '%s/%s' % (_("_Misc exercises"), _("_Scales")),
-    'toolbar': 4.5,
     'stock-icon': 'solfege-identify-scale',
+    'menupath': '/Menubar/FileMenu/MiscMenu',
+    'name': _("_Scales"),
+    'exercise_config': 'solfege/scales',
+    'action': 'start-exercise-scales',
 })
+
+import idbyname, dictation, twelvetone, \
+       idtone, compareintervals, singchord, rhythm, identifybpm, \
+       harmonicprogressiondictation, singanswer
+#import example
+
 exercise_setup.register_exercise({
     'exercise_name': 'id-by-name',
-    'exercise_config': 'solfege/scales-thirds',
-    'menu_path': '%s/%s' % (_("_Misc exercises"), _("Scales in t_hirds")),
     'stock-icon': 'solfege-identify-scale',
+    'menupath': '/Menubar/FileMenu/MiscMenu',
+    'name': _("Scales in t_hirds"),
+    'exercise_config': 'solfege/scales-thirds',
+    'action': 'start-exercise-scales-in-thirds',
+    'notoolbar': True,
 })
 
 
@@ -232,11 +224,12 @@ class DocViewer(gtk.VBox):
         We reread the docfile even if we are alreado displaying this file,
         just in case it has changed on disk.
         """
-        self.m_htmlwidget.read_docfile(fn)
+        if fn:
+            self.m_htmlwidget.read_docfile(fn)
         #if anchor:
         #    #FIXME jump_to_anchor is not implemented yet it gnome-python,
         #    # or maybe I just don't understand how to use it.
-        #    #self.m_htmlwidget.g_view.jump_to_anchor(anchor)
+        #    self.m_htmlwidget.g_view.jump_to_anchor(anchor)
         #else:
         a = self.m_htmlwidget.get_vadjustment()
         a.set_value(0.0)
@@ -266,7 +259,7 @@ class MusicViewerWindow(gtk.Dialog):
         self.g_music_displayer.display(music, fontsize)
 
 class MainWin(gnome.ui.App, cfg.ConfigUtils):
-    def __init__(self, argv):
+    def __init__(self, options):
         gnome.ui.App.__init__(self, 'solfege', 'GNU Solfege')
         self.icons = stock.SolfegeIconFactory(self, configureoutput.datadir)
         self.set_default_size(400, 400)
@@ -274,6 +267,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.set_icon(pixbuf)
         cfg.ConfigUtils.__dict__['__init__'](self, 'mainwin')
         self.set_resizable(self.get_bool('gui/mainwin_user_resizeable'))
+        self.add_watch('gui/mainwin_user_resizeable', lambda s: self.set_resizable(self.get_bool('gui/mainwin_user_resizeable')))
         self.connect('destroy', self.quit_program)
         self.connect('key_press_event', self.on_key_press_event)
         self.g_about_window = None
@@ -289,10 +283,110 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.m_key_bindings = {'history_back_ak': self.history_back,
                                'history_forward_ak': self.history_forward,
                                'history_reload_ak': self.history_reload}
-        self.m_app = app.SolfegeApp(argv, self)
-        self.setup_toolbar()
-        self.setup_menubar()
+        self.m_app = app.SolfegeApp(options, self)
+        self.setup_menu_and_toolbar()
         self.m_app.handle_href('welcome.html')
+    def setup_menu_and_toolbar(self):
+        self.g_action_group = gtk.ActionGroup('SolfegeActions')
+        self.g_action_group.add_actions([
+          ('FileMenu', None, _('_File')),
+          ('ChordsMenu', None, _('_Chords')),
+          ('IntervalsMenu', None, _('_Intervals')),
+          ('RhythmMenu', None, _('_Rhythm')),
+          ('MiscMenu', None, _('_Misc exercises')),
+          ('TheoryMenu', None, _('_Theory')),
+          ('TheoryIntervals', None, _('_Intervals'), None, None,
+            lambda o: self.m_app.handle_href('theory-intervals.html')),
+          ('ExerciseMenu', None, ''),
+          ('ViewMenu', None, _('_View')),
+          ('OpenPreferencesWindow', 'gtk-preferences', None, '<ctrl>F12', None,
+            self.open_preferences_window),
+          ('HelpMenu', None, _('_Help')),
+          ('HelpHelp', 'gtk-help', 'Help on current exercise', 'F1', None,
+            lambda o: self.m_app.please_help_me()),
+          ('HelpWelcome', None, _('_Welcome'), None, None,
+            lambda o: self.m_app.handle_href('welcome.html')),
+          ('HelpIndex', None, _('_All help files'), None, None,
+            lambda o: self.m_app.handle_href('toc.html')),
+          ('HelpAllLessonFiles', None, _('All installed _lesson files'), None,
+            None, lambda o: self.m_app.handle_href('solfege:all-lessonfiles')),
+          ('HelpCopyright', None, _('_Copyright notice'), None, None,
+            lambda o: self.m_app.handle_href('copyright.html')),
+          ('HelpCredits', None, _('C_redits'), None, None,
+            lambda o: self.m_app.handle_href('AUTHORS.html')),
+          ('HelpOnline', None, _('_Mailinglists, web page etc.'), None, None,
+            lambda o: self.m_app.handle_href('online-resources.html')),
+          ('HelpReportingBugs', None, _('Reporting _bugs'), None, None,
+            lambda o: self.m_app.handle_href('bug-reporting.html')),
+          ('HelpAbout', 'gtk-about', None, None, None, self.show_about_window),
+          ('AppQuit', 'gtk-quit', None, None, None, self.quit_program),
+          ('ViewZoomIn', 'gtk-zoom-in', _('Help viewer zoom in'), None, None,
+           lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()),
+          ('ViewZoomOut', 'gtk-zoom-out', _('Help viewer zoom out'), None, None,
+           lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_out()),
+          ('ViewZoom100', 'gtk-zoom-100', _('Help viewer zoom reset'), None, None,
+           lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_reset()),
+          ('BrowserGoBack', 'gtk-go-back', None, None, None, self.history_back),
+          ('BrowserGoForward', 'gtk-go-forward', None, None, None, self.history_forward),
+        ])
+        self.g_ui_manager = gtk.UIManager()
+        self.g_ui_manager.insert_action_group(self.g_action_group, 1)
+
+        self.g_ui_manager.add_ui_from_file("ui.xml")
+
+        a = gtk.ToggleAction('ToggleNavButtonsVisible', _('_Navigation buttons'), None, None)
+        if self.get_bool('gui/navbar_visible'):
+            a.set_active(True)
+        self.add_watch('gui/navbar_visible', self.on_watch_navbar_visible)
+        a.connect('activate', lambda w: self.set_bool('gui/navbar_visible', w.get_active()))
+        self.g_action_group.add_action(a)
+
+        a = gtk.ToggleAction('ToggleToolbarVisible', _('_Toolbar'), None, None)
+        if self.get_bool('gui/toolbar_visible'):
+            a.set_active(True)
+        self.add_watch('gui/toolbar_visible', self.on_watch_toolbar_visible)
+        a.connect('activate', lambda w: self.set_bool('gui/toolbar_visible', w.get_active()))
+        self.g_action_group.add_action(a)
+
+        if configureoutput.WITH_GTKHTML:
+            self.g_ui_manager.add_ui_from_string("""\
+<menubar name='Menubar'>
+ <menu action='ViewMenu'>
+      <menuitem action='ViewZoomIn'/>
+      <menuitem action='ViewZoomOut'/>
+      <menuitem action='ViewZoom100'/>
+ </menu>
+</menubar>""")
+
+        self.add_accel_group(self.g_ui_manager.get_accel_group())
+        for e in __exdata__:
+            if 'menupath' in e:
+                a = gtk.Action(e['action'], e['name'], e['exercise_pretty_name'], e['stock-icon'])
+                self.g_action_group.add_action(a)
+                a.connect('activate', lambda o, e=e:
+                    self.m_app.handle_href('solfege:/%s/%s' % (
+                      e['exercise_name'], e['exercise_config'])))
+                if not self.g_ui_manager.get_widget(e['menupath']):
+                    print "Miss:", e['menupath']
+                self.g_ui_manager.add_ui(self.g_ui_manager.new_merge_id(),
+                   e['menupath'], e['name'], e['action'],
+                   gtk.UI_MANAGER_MENUITEM, False)
+                if 'notoolbar' not in e:
+                    self.g_ui_manager.add_ui(self.g_ui_manager.new_merge_id(),
+                       '/Toolbar', e['name'], e['action'],
+                       gtk.UI_MANAGER_TOOLITEM, False)
+        self.set_menus(self.g_ui_manager.get_widget('/Menubar'))
+        self.on_watch_toolbar_style('gui/toolbar_style')
+        self.add_watch('gui/toolbar_style', self.on_watch_toolbar_style)
+        self.main_box.pack_start(self.g_ui_manager.get_widget('/Toolbar'), False)
+        if not self.get_bool('gui/toolbar_visible'):
+            self.g_ui_manager.get_widget('/Toolbar').hide()
+        if not self.get_bool('gui/navbar_visible'):
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').hide()
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').hide()
+        #FIXME not using GnomeApp.add_toolbar
+        #self.add_toolbar(self.g_ui_manager.get_widget('/Toolbar'),
+        #  'toolbaren', 'normal', 'top', 1, 0,0)
     def dialog_yesno(self, text):
         """Return True if the answer is yes, False if the answer is no.
         """
@@ -333,7 +427,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
             self.g_about_window = gnome.ui.About("GNU Solfege",
                   configureoutput.VERSION_STRING,
-                  "Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen",
+                  copyright,
                   _("GPL'ed ear training.") + "\n" + "http://www.solfege.org",
                   ["Tom Cato Amundsen",
                   'Oliver Vollmer %s' % _("(toolbar icons)"),
@@ -352,10 +446,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.g_about_window.destroy()
         self.g_about_window = None
     def post_constructor(self):
-        if not self.get_bool("gui/navbar_visible"):
-            self.get_dock_item_by_name('navbar').hide()
-        if not self.get_bool("gui/toolbar_visible"):
-            self.get_dock_item_by_name('toolbar').hide()
         self.box_dict['docviewer'].grab_focus()
         if self.m_app.m_sound_init_exception is not None:
             self.m_app.display_sound_init_error_message(self.m_app.m_sound_init_exception)
@@ -432,241 +522,35 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def open_preferences_window(self, widget=None):
         if not self.g_config_window:
             self.g_config_window = ConfigWindow(self.m_app)
-            #self.g_config_window.set_parent(self)
             self.g_config_window.show()
         else:
             self.g_config_window.show()
-    def setup_menubar(self):
-        ag = gtk.AccelGroup()
-        self.add_accel_group(ag)
-        self.g_menubar = menubar = gtk.MenuBar()
-        self.set_menus(self.g_menubar)
-        for label, func in ((_("_File"), self.create_file_menu),
-                            (_("_View"), self.create_view_menu),
-                            (_("_Help"), self.create_help_menu)):
-            menuitem = gtk.MenuItem(label)
-            menubar.add(menuitem)
-            menuitem.set_submenu(func(ag))
-        self.g_menubar.show_all()
-    def create_submenu(self, data, ag):
-        """FIXME rename
-        """
-        menu = gtk.Menu()
-        for d in data:
-            menuitem = gtk.ImageMenuItem(d['menu_path'].split('/')[1])
-            if 'icon' in d:
-                im = gtk.Image()
-                im.set_from_file(d['icon'])
-                im.set_from_pixbuf(im.get_pixbuf().scale_simple(16, 16, gtk.gdk.INTERP_TILES))
-                menuitem.set_image(im)
-            elif 'stock-icon' in d:
-                im = gtk.Image()
-                im.set_from_stock(d['stock-icon'], gtk.ICON_SIZE_MENU)
-                menuitem.set_image(im)
-            if 'exercise_config' in d:
-                s = 'solfege:practise/%s/%s' % (d['exercise_name'], d['exercise_config'])
-            else:
-                s = 'solfege:practise/%s' % d['exercise_name']
-            menuitem.connect('activate', lambda o, self=self, s=s: self.m_app.handle_href(s))
-            if 'accel_key' in d:
-                menuitem.add_accelerator('activate', ag, d['accel_key'], d['accel_mods'], gtk.ACCEL_VISIBLE)
-            menu.add(menuitem)
-        return menu
-    def create_ex_menu(self, exlist, ag):
-        """
-        Each item in exlist:
-        (MENUTEXT, URI, graphics/FILENAME.PNG)
-        The icon is optional.
-        """
-        menu = gtk.Menu()
-        for ex in exlist:
-            if ex[0] == ex[1] == 'SEP':
-                item = gtk.SeparatorMenuItem()
-            else:
-                item = gtk.ImageMenuItem(ex[0])
-                item.connect('activate', lambda o, url=ex[1]: self.m_app.handle_href(url))
-                if len(ex) > 2:
-                    im = gtk.Image()
-                    im.set_from_file(os.path.join('graphics', ex[2]))
-                    im.set_from_pixbuf(im.get_pixbuf().scale_simple(16, 16, gtk.gdk.INTERP_TILES))
-                    item.set_image(im)
-            menu.add(item)
-        return menu
-    def get_file_menu_data(self):
-        menus = {}
-        sections = []
-        for rec in __exdata__:
-            m, n = rec['menu_path'].split("/")
-            if m not in menus:
-                menus[m] = []
-                sections.append(m)
-            menus[m].append(rec)
-        return sections, menus
-    def create_file_menu(self, ag):
-        menu = gtk.Menu()
-        if self.get_bool("gui/menus_have_tearoff"):
-            item = gtk.TearoffMenuItem()
-            menu.add(item)
-        sections, d = self.get_file_menu_data()
-        for k in sections:
-            menuitem = gtk.MenuItem(k)
-            menu.add(menuitem)
-            menuitem.set_submenu(self.create_submenu(d[k], ag))
-        ##
-        menuitem = gtk.MenuItem(_("_Theory"))
-        menu.add(menuitem)
-        menuitem.set_submenu(self.create_ex_menu(
-              ( #(_('_Scale names'), 'theory-scales.html'),
-                (_('_Intervals'), 'theory-intervals.html'),
-               ), ag))
-        item = gtk.MenuItem()
-        item.add(gtk.HSeparator())
-        menu.add(item)
-        ##############
-        # Preferences
-        menu.add(gu.create_stock_menu_item('gtk-preferences', _("_Preferences"), self.open_preferences_window, ag, gtk.keysyms.F12, gtk.gdk.CONTROL_MASK))
-        #######
-        # Exit
-        menu.add(gu.create_stock_menu_item('gtk-quit', _("_Exit"), self.quit_program, ag, gtk.keysyms.Q, gtk.gdk.CONTROL_MASK))
-        return menu
-    def create_view_menu(self, ag):
-        menu = gtk.Menu()
-        if self.get_bool("gui/menus_have_tearoff"):
-            item = gtk.TearoffMenuItem()
-            menu.add(item)
-        # toolbar visible?
-        toolitem = gtk.CheckMenuItem(_("_Toolbar"))
-        toolitem.connect('toggled', self.on_view_menu_toolbar_visible_toggled)
-        toolitem.set_active(self.get_bool('gui/toolbar_visible'))
-        self.add_watch('gui/toolbar_visible', lambda key:
-          toolitem.set_active(self.get_bool('gui/toolbar_visible')))
-        menu.add(toolitem)
-
-        # navbar visible?
-        navitem = gtk.CheckMenuItem(_("_Navigation buttons"))
-        navitem.connect('toggled', self.on_view_menu_navbar_visible_toggled)
-        navitem.set_active(self.get_bool('gui/navbar_visible'))
-        self.add_watch('gui/navbar_visible', lambda key:
-          navitem.set_active(self.get_bool('gui/navbar_visible')))
-        menu.add(navitem)
-
-        if configureoutput.WITH_GTKHTML:
-            menu.add(gtk.SeparatorMenuItem())
-            menu.add(gu.create_stock_menu_item('gtk-zoom-in', _("Help viewer zoom in"),
-               lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in(),
-               ag, gtk.keysyms.plus, gtk.gdk.CONTROL_MASK))
-            menu.add(gu.create_stock_menu_item('gtk-zoom-out', _("Help viewer zoom out"),
-               lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_out(),
-               ag, gtk.keysyms.minus, gtk.gdk.CONTROL_MASK))
-            menu.add(gu.create_stock_menu_item('gtk-zoom-100', _("Help viewer zoom reset"),
-               lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_reset(),
-               ag, gtk.keysyms.equal, gtk.gdk.CONTROL_MASK))
-        return menu
     def on_zoom_in(self, *v):
         self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()
-    def on_view_menu_toolbar_visible_toggled(self, widget):
-        """
-        This function is called when the user selects the View/Toolbar menu
-        to change if the toolbar is visible or not.
-        """
-        self.set_bool('gui/toolbar_visible', widget.get_active())
-        if widget.get_active():
-            self.get_dock_item_by_name("toolbar").show()
+    def on_watch_toolbar_style(self, s):
+        assert self.get_string('gui/toolbar_style') in ('text', 'icons', 'both')
+
+        self.g_ui_manager.get_widget('/Toolbar').set_style({
+            'text': gtk.TOOLBAR_TEXT,
+            'icons': gtk.TOOLBAR_ICONS,
+            'both': gtk.TOOLBAR_BOTH}[self.get_string('gui/toolbar_style')])
+    def on_watch_toolbar_visible(self, s):
+        self.g_ui_manager.get_widget('/Menubar/ViewMenu/ToggleToolbarVisible').set_active(self.get_bool('gui/toolbar_visible'))
+        if self.get_bool('gui/toolbar_visible'):
+            self.g_ui_manager.get_widget('/Toolbar').show()
         else:
-            self.get_dock_item_by_name("toolbar").hide()
-    def on_view_menu_navbar_visible_toggled(self, widget):
-        """
-        This function is called when the user selects the View/Navbar menu
-        to change it the navbar is visible or not.
-        """
-        self.set_bool('gui/navbar_visible', widget.get_active())
-        if widget.get_active():
-            self.get_dock_item_by_name("navbar").show()
+            self.g_ui_manager.get_widget('/Toolbar').hide()
+    def on_watch_navbar_visible(self, widget):
+        self.g_ui_manager.get_widget('/Menubar/ViewMenu/ToggleNavButtonsVisible').set_active(self.get_bool('gui/navbar_visible'))
+        if self.get_bool('gui/navbar_visible'):
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').show()
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').show()
         else:
-            self.get_dock_item_by_name("navbar").hide()
-    def create_help_menu(self, ag):
-        menu = gtk.Menu()
-        if self.get_bool("gui/menus_have_tearoff"):
-            item = gtk.TearoffMenuItem()
-            menu.add(item)
-        item = gu.create_stock_menu_item('gtk-help', _("_Help on current exercise"),
-                            lambda o, self=self: self.m_app.please_help_me(),
-                            ag, gtk.keysyms.F1, 0)
-        menu.add(item)
-        for txt, url in (
-                     (_("_Welcome"), 'welcome.html'),
-                     (_("_All help files"), 'toc.html'),
-                     (_("All installed _lesson files"), 'solfege:all-lessonfiles'),
-                     ('SEP', 'SEP'),
-                     (_("_Copyright notice"), 'copyright.html'),
-                     (_("C_redits"), 'AUTHORS.html'),
-                     (_("_Mailinglists, web page etc."), 'online-resources.html'),
-                     (_("Reporting _bugs"), 'bug-reporting.html'),
-                     ):
-            if txt == url == 'SEP':
-                item = gtk.SeparatorMenuItem()
-                menu.add(item)
-            else:
-                item = gtk.MenuItem(txt)
-                item.connect('activate', lambda o, self=self, url=url: self.m_app.handle_href(url))
-                menu.add(item)
-        item = gtk.MenuItem(_("_About"))
-        item.connect('activate', self.show_about_window)
-        menu.add(item)
-        return menu
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').hide()
+            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').hide()
     def quit_program(self, w=None):
         self.m_app.quit_program()
         gtk.main_quit()
-    def setup_toolbar(self):
-        ## navbar
-        self.g_navbar = gtk.Toolbar()
-        self.g_navbar.insert_stock('gtk-go-forward', None, None,
-                                   self.history_forward, 0, 0)
-        self.g_navbar.insert_stock('gtk-go-back', None, None,
-                                   self.history_back, 0, 0)
-        self.g_navbar.show_all()
-        self.add_toolbar(self.g_navbar, "navbar", "normal", "top", 1, 0, 0)
-        s = self.get_string('gui/navbar_style')
-        if not (s in ('icons', 'text', 'both')):
-            s = 'icons'
-            self.set_string('gui/navbar_style', 'icons')
-        self.g_navbar.set_style({'icons': gtk.TOOLBAR_ICONS,
-                                 'text': gtk.TOOLBAR_TEXT,
-                                 'both': gtk.TOOLBAR_BOTH}[s])
-        if self.get_bool('gui/navbar_visible'):
-            self.get_dock_item_by_name("navbar").show()
-        else:
-            self.get_dock_item_by_name("navbar").hide()
-        # main toolbar
-        self.g_toolbar =  gtk.Toolbar()
-        exdict = {}
-        tblist_ordered = []
-        tblist_unordered = []
-        for e in __exdata__:
-            exdict[e['exercise_name']] = e
-            if 'toolbar' in e:
-                tblist_ordered.append(e)
-        tblist_ordered.sort(lambda a, b: cmp(a['toolbar'], b['toolbar']))
-        for ex in tblist_ordered + tblist_unordered:
-            im = gtk.Image()
-            if 'toolbar' not in ex:
-                continue
-            if ex.has_key('icon'):
-                im.set_from_file(ex['icon'])
-            else:
-                assert ex.has_key('stock-icon')
-                im.set_from_stock(ex['stock-icon'], gtk.ICON_SIZE_SMALL_TOOLBAR)
-            button = self.g_toolbar.append_item(ex['exercise_pretty_name'], ex['exercise_pretty_name'], None, im, None, None)
-            button.connect('clicked', lambda button, ex=ex: self.m_app.handle_href("solfege:practice/"+ex['exercise_name']+'/'+ex['exercise_config']))
-        self.add_toolbar(self.g_toolbar, "toolbar", "normal", "top", 1, 0, 1)
-        s = self.get_string('gui/toolbar_style')
-        if not (s in ('icons', 'text', 'both')):
-            s = 'icons'
-            self.set_string('gui/toolbar_style', 'icons')
-        self.g_toolbar.set_style({'icons': gtk.TOOLBAR_ICONS,
-                                  'text': gtk.TOOLBAR_TEXT,
-                                  'both': gtk.TOOLBAR_BOTH}[s])
-        self.g_toolbar.show_all()
     def display_in_musicviewer(self, music):
         if not self.g_musicviewer_window:
             self.g_musicviewer_window = MusicViewerWindow(self)
@@ -676,13 +560,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.g_musicviewer_window.destroy()
         self.g_musicviewer_window = None
 
-#UGH is this really necessary
-def create_menu_item(txt, callback, ag, accel_key, accel_mod):
-    item = gtk.MenuItem(txt)
-    item.add_accelerator('activate', ag, accel_key, accel_mod, gtk.ACCEL_VISIBLE)
-    item.connect('activate', callback)
-    return item
-
 if splash_win:
     splash_win.show_progress("running gnome.program_init")
 
@@ -693,10 +570,12 @@ prog.parse_args()
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
 
+cfg.set_bool('config/no_random', bool(options.no_random))
+
 if splash_win:
     splash_win.show_progress("creating MainWin")
 
-w = MainWin(args)
+w = MainWin(options)
 w.show()
 if splash_win:
     splash_win.destroy()
